@@ -1,10 +1,30 @@
-/* Jeroen Ooms (2018)
+/* Jeroen Ooms (2020)
  * Bindings to vectorized image manipulations.
  * See API: https://www.imagemagick.org/Magick++/STL.html
  */
 
 #include "magick_types.h"
 
+// [[Rcpp::init]]
+void my_magick_init(DllInfo *dll) {
+ // Workaround for sMacOS OpenMP conflict: https://github.com/ropensci/magick/issues/170
+#ifdef __APPLE__
+  setenv("KMP_DUPLICATE_LIB_OK", "1", 1);
+#endif
+  Magick::InitializeMagick("");
+}
+
+// [[Rcpp::export]]
+int magick_threads(size_t i = 0){
+#if MagickLibVersion >= 0x689
+  if(i > 0)
+    Magick::ResourceLimits::thread(i);
+  return Magick::ResourceLimits::thread();
+#else
+  return NA_INTEGER;
+#endif
+}
+
 //External R pointer finalizer
 void finalize_image( Image *image ){
   delete image;
@@ -37,9 +57,18 @@ XPtrImage create (){
 }
 
 // [[Rcpp::export]]
-XPtrImage magick_image_blank(size_t width, size_t height, const char * color, const char * pseudo_image){
+XPtrImage magick_image_blank(size_t width, size_t height, const char * color,
+                             const char * pseudo_image, SEXP options){
   Frame x(Geom(width, height), Color(color));
   x.magick("png");
+  if(Rf_length(options)){
+    SEXP names = Rf_getAttrib(options, R_NamesSymbol);
+    for(int i = 0; i < Rf_length(options); i++){
+      const char *name = CHAR(STRING_ELT(names, i));
+      const char *value = CHAR(STRING_ELT(options, i));
+      MagickCore::SetImageOption(x.imageInfo(), name, value);
+    }
+  }
   if(strlen(pseudo_image))
     x.read(pseudo_image);
   XPtrImage image = create(1);
@@ -136,3 +165,17 @@ bool autobrewed(){
   return false;
 #endif
 }
+
+
+// [[Rcpp::export]]
+Rcpp::List magick_resource_limits(){
+  return Rcpp::List::create(
+#if MagickLibVersion >= 0x689
+    Rcpp::_["area"] = Magick::ResourceLimits::area(),
+    Rcpp::_["thread"] = Magick::ResourceLimits::thread(),
+    Rcpp::_["map"] = Magick::ResourceLimits::map(),
+    Rcpp::_["memory"] = Magick::ResourceLimits::memory(),
+    Rcpp::_["disk"] = Magick::ResourceLimits::disk()
+#endif
+  );
+}
